%______________________________________________________________________________________________
%  Moth-Flame Optimization Algorithm (MFO)                                                            
%  Source codes demo version 1.0                                                                      
%                                                                                                     
%  Developed in MATLAB R2011b(7.13)                                                                   
%                                                                                                     
%  Author and programmer: Seyedali Mirjalili                                                          
%                                                                                                     
%         e-Mail: ali.mirjalili@gmail.com                                                             
%                 seyedali.mirjalili@griffithuni.edu.au                                               
%                                                                                                     
%       Homepage: http://www.alimirjalili.com                                                         
%                                                                                                     
%  Main paper:                                                                                        
%  S. Mirjalili, Moth-Flame Optimization Algorithm: A Novel Nature-inspired Heuristic Paradigm, 
%  Knowledge-Based Systems, DOI: http://dx.doi.org/10.1016/j.knosys.2015.07.006
%_______________________________________________________________________________________________

% This function containts full information and implementations of the benchmark 
% functions in Table 1, Table 2, and other test functins from the literature 

% lb is the lower bound: lb=[lb_1,lb_2,...,lb_d]
% up is the uppper bound: ub=[ub_1,ub_2,...,ub_d]
% dim is the number of variables (dimension of the problem)

function [lb,ub,dim,fobj] = Get_Functions_details(F)
D=30;

switch F
     case 'F30' %solar1
        fobj = @F30;
        lb=[0,0, 0,0,1];
        ub=[0.5,100,1,1,2];
        dim=5;
    case 'F31'%solar2
        fobj = @F31;
        dim=7;
        lb=[0,0,1,1,0,0,0];
        ub=[1,1,2,2,0.5,100,1];
    case 'PV1'
        fobj = @PV1;
        lb=[0,0, 0,0,1];
        ub=[0.5,100,1,1,2];
        dim=5;
    case 'PV2'
        fobj = @PV2;
        dim=7;
        lb=[0,0,1,1,0,0,0];
        ub=[1,1,2,2,0.5,100,1];
    case 'F1'
        fobj = @F1;
        lb=-100;
        ub=100;
        dim=D;    
    case 'F2'
        fobj = @F2;
        lb=-10;
        ub=10;
        dim=D;
        
    case 'F3'
        fobj = @F3;
        lb=-100;
        ub=100;
        dim=D;
        
    case 'F4'
        fobj = @F4;
        lb=-100;
        ub=100;
        dim=D;
        
    case 'F5'
        fobj = @F5;
        lb=-30;
        ub=30;
        dim=D;
        
    case 'F6'
        fobj = @F6;
        lb=-100;
        ub=100;
        dim=D;
        
    case 'F7'
        fobj = @F7;
        lb=-1.28;
        ub=1.28;
        dim=D;
        
    case 'F8'
        fobj = @F8;
        lb=-500;
        ub=500;
        dim=D;
        
    case 'F9'
        fobj = @F9;
        lb=-5.12;
        ub=5.12;
        dim=D;
        
    case 'F10'
        fobj = @F10;
        lb=-32;
        ub=32;
        dim=D;
        
    case 'F11'
        fobj = @F11;
        lb=-600;
        ub=600;
        dim=D;
        
    case 'F12'
        fobj = @F12;
        lb=-50;
        ub=50;
        dim=D;
        
    case 'F13'
        fobj = @F13;
        lb=-50;
        ub=50;
        dim=D;
        
    case 'F14'
        fobj = @F14;
        lb=-65.536;
        ub=65.536;
        dim=2;
        
    case 'F15'
        fobj = @F15;
        lb=-5;
        ub=5;
        dim=4;
        
    case 'F16'
        fobj = @F16;
        lb=-5;
        ub=5;
        dim=2;
        
    case 'F17'
        fobj = @F17;
        lb=[-5,0];
        ub=[10,15];
        dim=2;
        
    case 'F18'
        fobj = @F18;
        lb=-2;
        ub=2;
        dim=2;
        
    case 'F19'
        fobj = @F19;
        lb=0;
        ub=1;
        dim=3;
        
    case 'F20'
        fobj = @F20;
        lb=0;
        ub=1;
        dim=6;     
        
    case 'F21'
        fobj = @F21;
        lb=0;
        ub=10;
        dim=4;    
        
    case 'F22'
        fobj = @F22;
        lb=0;
        ub=10;
        dim=4;    
        
    case 'F23'
        fobj = @F23;
        lb=0;
        ub=10;
        dim=4;            
end

end

% F1

function lse = F30(x)
        load curr;
        load volt;
        VT = 1.3806e-23*(273.15+33)/1.602e-19;
        Rs = x(1,1);
        Rsh = x(1,2);
        Iph = x(1,3); 
        Isd = x(1,4);
        n = x(1,5);
        lse = 0;
        len = length(volt);
        %Calculate the lse
        for m = 1:len
%         answer=bisect(@(x)((Iph -Isd *1e-6*(exp((volt(m,1)+ Rs*x)/(n*VT)) - 1 ) - (volt(m,1)+ Rs*x)/Rsh)-x),-1,1,1e-8);
        answer  = (Iph -Isd *1e-6*(exp((volt(m,1)+ Rs*curr(m,1))/(n*VT)) - 1 ) - (volt(m,1)+ Rs*curr(m,1))/Rsh);
        currcal(m,1)=answer;
        lse = lse + (answer-curr(m,1))^2;
        end
        lse = (lse/len)^0.5;
end
function lse = F31(x)
        load curr;
        load volt;
        VT = 1.3806e-23*(273.15+33)/1.602e-19;
        Isd1 = x(1,1);
        Isd2 = x(1,2);
        n1 = x(1,3);
        n2 = x(1,4);
        Rs = x(1,5); 
        Rsh = x(1,6);
        Iph = x(1,7);
        lse = 0;
        len = length(volt);
        %Calculate the lse
        for m = 1:len
%         CurrCal(m,1)  = (Iph -Isd *1e-6*(exp((volt(m,1)+ Rs*curr(m,1))/(n*VT)) - 1 ) - (volt(m,1)+ Rs*curr(m,1))/Rsh);
%         lse = lse + (CurrCal(m,1)-curr(m,1))^2;
%          answer=bisect(@(x)((Iph -Isd1 *1e-6*(exp((volt(m,1)+ Rs*x)/(n1*VT)) - 1 ) - Isd2 *1e-6*(exp((volt(m,1)+ Rs*x)/(n2*VT)) - 1 )-(volt(m,1)+ Rs*x)/Rsh)-x),-1,1,1e-8);
%         currcal(m,1)=answer;
        CurrCal(m,1)  = Iph -Isd1 *1e-6*(exp((volt(m,1)+ Rs*curr(m,1))/(n1*VT)) - 1 ) - Isd2 *1e-6*(exp((volt(m,1)+ Rs*curr(m,1))/(n2*VT)) - 1 )-(volt(m,1)+ Rs*curr(m,1))/Rsh;
%         lse = lse + (answer-curr(m,1))^2;
        lse = lse + (CurrCal(m,1)-curr(m,1))^2;
        end
        lse = (lse/len)^0.5;
end

function answer=bisect(f,a,b,delta)

%Input - f is the function 
% - a and b are the left and right endpoints 
% - delta is the tolerance 
%Output - c is the zero 
% - yc= f(c) 
% - err is the error estimate for c 

%If f is defined as an M-file function use the @ notation 
% call [c,err,yc]=bisect(@f,a,b,delta). 
%If f is defined as an anonymous function use the 
% call [c,err,yc]=bisect(f,a,b,delta). 

% NUMERICAL METHODS: Matlab Programs 
% (c) 2004 by John H. Mathews and Kurtis D. Fink 
% Complementary Software to accompany the textbook: 
% NUMERICAL METHODS: Using Matlab, Fourth Edition 
% ISBN: 0-13-065248-2 
% Prentice-Hall Pub. Inc. 
% One Lake Street 
% Upper Saddle River, NJ 07458 

ya=f(a); 
yb=f(b); 
if ya*yb > 0
    answer=(a+b)/2;
    return;
end 
max1=1+round((log(b-a)-log(delta))/log(2)); 
for k=1:max1 
c=(a+b)/2; 
yc=f(c); 
if yc==0 
a=c; 
b=c; 
elseif yb*yc>0 
b=c; 
yb=yc; 
else 
a=c; 
ya=yc; 
end 
if b-a < delta, break,end 
end
answer=(a+b)/2; 
err=abs(b-a); 
yc=f(c);
end

% F1
function o = F1(x)  % Sphere model
    [ps,D]=size(x); 
    o=sum(x.^2,2);
end

function o = F2(x)  % Schwefels problem 2.22
    [ps,D]=size(x);
    o=sum(abs(x),2)+prod(abs(x),2); 
end

function o = F3(x)  % Schwefels problem 1.2
    [ps,D]=size(x);
    for k=1:ps
        temp=cumsum(x(k,:));
        temp=temp.^2;
        o(k,:)=sum(temp); 
    end
end

function o = F4(x)  % Schwefels problem 2.21
    [ps,D]=size(x);
    for k=1:ps
        temp=abs(x(k,:));
        o(k,:)=max(temp); 
    end
end

function o = F5(x)  % Generalized Rosenbrocks functions
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        for i=1:D-1
            o(k,:)=o(k,:)+100*(x(k,i+1)-x(k,i)^2)^2+(1-x(k,i))^2; 
        end
    end
end

function o = F6(x)  % Step Function
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        temp = floor(x+0.5);
        o=sum(temp.^2,2);
    end
end

function o = F7(x)  % Quartic Function i.e. Niose
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        for i=1:D
            o(k,:)=o(k,:)+i*x(k,i)^4; 
        end
        o(k,:)=o(k,:)+rand;
    end
end

function o = F8(x)  % Generalized Schwefel's Problem 2.26
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        for i=1:D
            o(k,:)=o(k,:)-x(k,i)*sin(sqrt(abs(x(k,i)))); 
        end
    end
end

function o = F9(x)  % Generalized Rastrigin's Function
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        for i=1:D
            o(k,:)=o(k,:)+x(k,i)^2-10*cos(2*pi*x(k,i))+10; 
        end
    end
end

function o = F10(x)  % Ackley's Function
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        temp1=(sum(x(k,:).^2))/D;
        temp1=-20*exp(-0.2*sqrt(temp1));
        temp2=sum(cos(2*pi*x(k,:)))/D;
        temp2=-exp(temp2); 
        o(k,:)=o(k,:)+temp1+20+temp2+exp(1); 
    end
end

function o = F11(x)  % Generalized Griewank Function
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=sum(x(k,:).^2)/4000-prod(cos(x(k,:)./(sqrt([1:D]))))+1; 
    end
end

function o = F12(x)  % Generalized Penalized Function 1
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        y(k,:)=1+(x(k,:)+1)./4;
        temp1=0;
        for i=1:D-1
            temp1=temp1+(y(k,i)-1)^2*(1+10*(sin(pi*y(k,i+1)))^2);
        end
        temp1=temp1+10*(sin(pi*y(k,1)))^2+(y(k,D)-1)^2;
        temp1=temp1*pi/D;
        temp2=0;
        for i=1:D
            temp2=temp2+u12(x(k,i),10,100,4);
        end
        o(k,:)=o(k,:)+temp1+temp2;
    end
end

function o = F13(x)  % Generalized Penalized Function 2
    [ps,D]=size(x);
    for k=1:ps
        o(k,:)=0;
        temp1=0;
        for i=1:D-1
            temp1=temp1+(x(k,i)-1)^2*(1+(sin(3*pi*x(k,i+1)))^2);
        end
        temp1=temp1+(sin(3*pi*x(k,1)))^2+(x(k,D)-1)^2;
        temp1=temp1*0.1;
        temp2=0;
        for i=1:D
            temp2=temp2+u12(x(k,i),5,100,4);
        end
        o(k,:)=o(k,:)+temp1+temp2;
    end
end

% F14

function o = F14(x)
aS=[-32 -16 0 16 32 -32 -16 0 16 32 -32 -16 0 16 32 -32 -16 0 16 32 -32 -16 0 16 32;,...
-32 -32 -32 -32 -32 -16 -16 -16 -16 -16 0 0 0 0 0 16 16 16 16 16 32 32 32 32 32];

for j=1:25
    bS(j)=sum((x'-aS(:,j)).^6);
end
o=(1/500+sum(1./([1:25]+bS))).^(-1);
end

% F15

function o = F15(x)
aK=[.1957 .1947 .1735 .16 .0844 .0627 .0456 .0342 .0323 .0235 .0246];
bK=[.25 .5 1 2 4 6 8 10 12 14 16];bK=1./bK;
o=sum((aK-((x(1).*(bK.^2+x(2).*bK))./(bK.^2+x(3).*bK+x(4)))).^2);
end

% F16

function o = F16(x)
o=4*(x(1)^2)-2.1*(x(1)^4)+(x(1)^6)/3+x(1)*x(2)-4*(x(2)^2)+4*(x(2)^4);
end

% F17

function o = F17(x)
o=(x(2)-(x(1)^2)*5.1/(4*(pi^2))+5/pi*x(1)-6)^2+10*(1-1/(8*pi))*cos(x(1))+10;
end

% F18

function o = F18(x)
o=(1+(x(1)+x(2)+1)^2*(19-14*x(1)+3*(x(1)^2)-14*x(2)+6*x(1)*x(2)+3*x(2)^2))*...
    (30+(2*x(1)-3*x(2))^2*(18-32*x(1)+12*(x(1)^2)+48*x(2)-36*x(1)*x(2)+27*(x(2)^2)));
end

% F19

function o = F19(x)
aH=[3 10 30;.1 10 35;3 10 30;.1 10 35];cH=[1 1.2 3 3.2];
pH=[.3689 .117 .2673;.4699 .4387 .747;.1091 .8732 .5547;.03815 .5743 .8828];
o=0;
for i=1:4
    o=o-cH(i)*exp(-(sum(aH(i,:).*((x-pH(i,:)).^2))));
end
end

% F20

function o = F20(x)
aH=[10 3 17 3.5 1.7 8;.05 10 17 .1 8 14;3 3.5 1.7 10 17 8;17 8 .05 10 .1 14];
cH=[1 1.2 3 3.2];
pH=[.1312 .1696 .5569 .0124 .8283 .5886;.2329 .4135 .8307 .3736 .1004 .9991;...
.2348 .1415 .3522 .2883 .3047 .6650;.4047 .8828 .8732 .5743 .1091 .0381];
o=0;
for i=1:4
    o=o-cH(i)*exp(-(sum(aH(i,:).*((x-pH(i,:)).^2))));
end
end

% F21

function o = F21(x)
aSH=[4 4 4 4;1 1 1 1;8 8 8 8;6 6 6 6;3 7 3 7;2 9 2 9;5 5 3 3;8 1 8 1;6 2 6 2;7 3.6 7 3.6];
cSH=[.1 .2 .2 .4 .4 .6 .3 .7 .5 .5];

o=0;
for i=1:5
    o=o-((x-aSH(i,:))*(x-aSH(i,:))'+cSH(i))^(-1);
end
end

% F22

function o = F22(x)
aSH=[4 4 4 4;1 1 1 1;8 8 8 8;6 6 6 6;3 7 3 7;2 9 2 9;5 5 3 3;8 1 8 1;6 2 6 2;7 3.6 7 3.6];
cSH=[.1 .2 .2 .4 .4 .6 .3 .7 .5 .5];

o=0;
for i=1:7
    o=o-((x-aSH(i,:))*(x-aSH(i,:))'+cSH(i))^(-1);
end
end

% F23

function o = F23(x)
aSH=[4 4 4 4;1 1 1 1;8 8 8 8;6 6 6 6;3 7 3 7;2 9 2 9;5 5 3 3;8 1 8 1;6 2 6 2;7 3.6 7 3.6];
cSH=[.1 .2 .2 .4 .4 .6 .3 .7 .5 .5];

o=0;
for i=1:10
    o=o-((x-aSH(i,:))*(x-aSH(i,:))'+cSH(i))^(-1);
end
end

function o=Ufun(x,a,k,m)
o=k.*((x-a).^m).*(x>a)+k.*((-x-a).^m).*(x<(-a));
end

function u=u12(x,a,k,m)
% f12: Generalized Penalized Function
if x>a
    u=k*(x-a)^m; 
elseif x>=-a & x<=a
    u=0;
elseif x<a
    u=k*(-x-a)^m;
end  
end